/*
  printer.cpp - Library for flashing Morse code.
  Created by David A. Mellis, November 2, 2007.
  Released into the public domain.
*/

#include "Arduino.h"
#include "printer.h"


const byte GET_PAPER_STATUS = 0x12;
const byte START_GRAPHIC_MODE = 0x11;
const byte PRINT_SERIAL_NUMBER = 0x13;
const byte PRINT_DEVICE_INFO = 0x14;
const byte PRINT_BUFFER = 0x0A;

// Height in dots of a printer line
const byte LINE_HEIGHT_DOTS = 7;

// Width in dots of a printer line
const byte LINE_WIDTH_DOTS = 105;

// Dead distance in dots betwheen two lines
const byte INTERLINE_DEAD_DOTS = 4;

// Lenght in characters of a printer line
const byte CHARACTERS_PER_LINE = 15;

// Single character width in dots
const byte CHARACTER_WIDTH_DOTS = 5;

// Single character height in dots
const byte CHARACTER_HEIGHT_DOTS = LINE_HEIGHT_DOTS;

// Dots betwheen characters
const byte CHARACTER_DISTANCE_DOTS = 2;




Printer::Printer()
{

}

// Open PrinterShield's communication channel
bool Printer::Open()
{
    // Open serial communications and wait for port to open:
    Serial.begin(115200);
    return true;
}

// Close PrinterShield's communications channel
bool Printer::Close()
{
    // Close serial port
    Serial.end();
    return true;
}

// Print some text
bool Printer::PrintLine(String line)
{
    WriteByte(0xD); // Do carriage return (reset internal cursor)

    for (int x = 0; x < line.length(); x++)
    {
        WriteByte(line[x]);
    }

    PrintBuffer();  // Print current buffer data

    return !ManagePaperStatus();
}


// Print a graphic line (7x105 pixels)
bool Printer::PrintImage(byte* img)
{
    StartGraphicMode();
    for (int x = 0; x < LINE_WIDTH_DOTS; x++)
    {
        WriteByte(img[x]);
    }
    return !ManagePaperStatus();
}

// Print firmware device info
bool Printer::PrintDeviceInfos()
{
    WriteByte(PRINT_DEVICE_INFO);
    return !ManagePaperStatus();
}

// Print the full set of characters
bool Printer::PrintFullCharset()
{
    for (int x = 0; x <= 255; x++)
    {
        if (x >= 8 && x <= 20) { continue; }
        WriteByte((byte)x);
    }
    PrintBuffer();
    return !ManagePaperStatus();
}

// Prints current buffered data
bool Printer::PrintBuffer()
{
    WriteByte(PRINT_BUFFER);
}

// Get paper status
bool Printer::GetPaperStatus()
{
    return WriteByte(GET_PAPER_STATUS) != 0;
}

// Reads printer serial number
bool Printer::GetSerialNumber(byte* result)
{
    WriteByte(PRINT_SERIAL_NUMBER);

    for (int x = 0; x < 15; x++)
    {
        result[x] = (byte)Serial.read();
    }

    return true;
}


bool Printer::ManagePaperStatus()
{
    bool currentStatus = GetPaperStatus();
    if (_lastPaperStatus != currentStatus)
    {
        if (currentStatus)
        {
            if (PaperBlocked != NULL)
            {
                PaperBlocked(this);
            }
        }
        else
        {
            if (PaperBlocked != NULL)
            {
                PaperRestored(this);
            }
        }
    }
    _lastPaperStatus = currentStatus;
    return currentStatus;
}

void Printer::StartGraphicMode()
{
    WriteByte(START_GRAPHIC_MODE);
}

byte Printer::WriteByte(byte data)
{
    // Empty buffer
    while (Serial.available() > 0)
    {
        Serial.read();
    }

    // Write a byte
    Serial.write(data);

    // Wait a reply for about 10 seconds (measured 8, LOL)
    for (int x; x < 10000; x++)
    {
        if (Serial.available())
        {
            return Serial.read();
        }
        delay(1);
    }
    return 0;
}

